package http

import (
	"io/ioutil"
	"net/http"
	"testing"
	"time"

	"bitbucket.org/orus-io/xbus/fullenv_tests"
	"github.com/stretchr/testify/assert"
)

func TestFullenv(t *testing.T) {
	env, _ := fullenv.NewFullEnvForTest(t, "fullenv", "fullenv_test.yaml")
	defer env.Close()
	env.Init()
	if err := env.Startup(); err != nil {
		t.Fatal(err)
	}
	defer env.Shutdown()
	time.Sleep(250 * time.Millisecond)
	resp, err := http.Get("http://localhost:8911/test/processrequest")
	if err != nil {
		t.Fatal(err)
	}
	if resp.StatusCode != 404 {
		t.Fatal("Expected a 404 error")
	}
	resp, err = http.Get("http://localhost:8911/test/processrequest?key=test")
	if err != nil {
		t.Fatal(err)
	}
	if resp.StatusCode != 404 {
		t.Fatal("Expected a 404 error")
	}
	data, err := ioutil.ReadAll(resp.Body)
	assert.Equal(t, `{"error": "nats: authorization violation"}`, string(data))
}
